//
//  MainWindowController.m
//  CoreDataLibraryAppOSX
//
//  Created by Duncan Groenewald on 12/02/2014.
//  Copyright (c) 2014 OSSH Pty Ltd. All rights reserved.
//

#import "MainWindowController.h"
#import "OSxCDStackManager.h"

@interface MainWindowController ()

@end

@implementation MainWindowController

- (id)initWithWindow:(NSWindow *)window
{
    self = [super initWithWindow:window];
    if (self) {
        // Initialization code here.
        [self initialiseSidebarMenuItems];
        _taskCount = 0;
        _storeIsReady = NO;

    }
    return self;
}

- (void)windowDidLoad
{
    [super windowDidLoad];
    
    // Implement this method to handle any initialization after your window controller's window has been loaded from its nib file.
    //set the start menu option
    [_sidebarOutlineView sizeLastColumnToFit];
    [_sidebarOutlineView reloadData];
    [_sidebarOutlineView setFloatsGroupRows:NO];
    
    // NSTableViewRowSizeStyleDefault should be used, unless the user has picked an explicit size. In that case, it should be stored out and re-used.
    [_sidebarOutlineView setRowSizeStyle:NSTableViewRowSizeStyleDefault];
    
    // Expand all the root items; disable the expansion animation that normally happens
    [NSAnimationContext beginGrouping];
    [[NSAnimationContext currentContext] setDuration:0];
    [_sidebarOutlineView expandItem:nil expandChildren:YES];
    [NSAnimationContext endGrouping];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(storeReady) name:OSStoreReadyNotification object:[OSxCDStackManager sharedManager]];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(setUpgradeView:) name:OSStoreUpgradingNotification object:[OSxCDStackManager sharedManager]];

    [[OSxCDStackManager sharedManager] checkUserICloudPreferenceAndSetupIfNecessary];

}
- (void)storeReady {
    FLOG(@" called");
    _storeIsReady = YES;
    //[self performSelectorOnMainThread:@selector(setContentViewToMain) withObject:nil waitUntilDone:NO];
}
- (void)close {
    FLOG(@" called");
    [self saveWindowPosition];
}
- (void)initialiseSidebarMenuItems {
    _topLevelItems = [NSArray arrayWithObjects:@"Companies & Contacts", @"Finance", @"Quality", @"IT Systems", @"Administration", nil];
    
    _childrenDictionary = [NSMutableDictionary new];
    
    [_childrenDictionary setObject:[NSArray arrayWithObjects:
                                    [[OSSideBarMenuItem alloc] initWith:@"Companies"],
                                    [[OSSideBarMenuItem alloc] initWith:@"People"],
                                    [[OSSideBarMenuItem alloc] initWith:@"Contacts"],
                                    [[OSSideBarMenuItem alloc] initWith:@"Projects"],
                                    nil] forKey:@"Companies & Contacts"];
    
    [_childrenDictionary setObject:[NSArray arrayWithObjects:
                                    [[OSSideBarMenuItem alloc] initWith:@"Issues"],
                                    [[OSSideBarMenuItem alloc] initWith:@"Risks"],
                                    nil] forKey:@"Quality"];
    
    [_childrenDictionary setObject:[NSArray arrayWithObjects:
                                    [[OSSideBarMenuItem alloc] initWith:@"Expenses"],
                                    [[OSSideBarMenuItem alloc] initWith:@"Invoices"],
                                    [[OSSideBarMenuItem alloc] initWith:@"Statements"],
                                    nil] forKey:@"Finance"];
    
    [_childrenDictionary setObject:[NSArray arrayWithObjects:
                                    [[OSSideBarMenuItem alloc] initWith:@"Applications"],
                                    [[OSSideBarMenuItem alloc] initWith:@"Interfaces"],
                                    nil] forKey:@"IT Systems"];
    
    [_childrenDictionary setObject:[NSArray arrayWithObjects:
                                    [[OSSideBarMenuItem alloc] initWith:@"Backups"],
                                    [[OSSideBarMenuItem alloc] initWith:@"iCloud"],
                                    nil] forKey:@"Administration"];
    
}
- (NSImage*)itemImage:(id)item
{
    //LOG(@"itemImage called");
    
    NSImage *image = nil;
    if (item != nil) {
        image = [NSImage imageNamed:[NSString stringWithFormat:@"%@MenuIcon.png",item]];
    }
    else
        image = [NSImage imageNamed:@"MenuIcon.png"];
    
    if (image == nil)
        image = [NSImage imageNamed:@"MenuIcon.png"];
    
    return image;
}
// Shares in iCloud or removes from iCloud
- (IBAction)shareAction:(id)sender
{
    FLOG(@" called");
    
    NSMenuItem *menuItem = (NSMenuItem *)sender;
    if (_currentContentViewController) {
        [[_currentContentViewController view] removeFromSuperview];
    }
    _currentContentViewController = nil;
    
    if ([menuItem.title isEqualToString:@"Remove from iCloud"]) {
        FLOG(@" remove from iCloud");
        [self performSelectorOnMainThread:@selector(setICloudNo) withObject:nil waitUntilDone:NO];
    } else {
        FLOG(@" share in iCloud");
        [self performSelectorOnMainThread:@selector(setICloudYes) withObject:nil waitUntilDone:NO];
    }
    
    FLOG(@" finished");
    
}

- (void)setICloudYes {
    [[OSxCDStackManager sharedManager] setICloudPreference:YES];
}
- (void)setICloudNo {
    [[OSxCDStackManager sharedManager] setICloudPreference:NO];
}
// Performs the save action for the application, which is to send the save: message to the application's managed object context. Any encountered errors are presented to the user.
- (IBAction)saveAction:(id)sender
{
    FLOG(@"%@:%@ saveAction called", [self class], NSStringFromSelector(_cmd));
    NSError *error = nil;
    
    if (![[[OSxCDStackManager sharedManager] managedObjectContext] commitEditing]) {
        FLOG(@"%@:%@ unable to commit editing before saving", [self class], NSStringFromSelector(_cmd));
    }
    
    if (![[[OSxCDStackManager sharedManager] managedObjectContext] save:&error]) {
        [[NSApplication sharedApplication] presentError:error];
    }
}
- (IBAction)makebackupAction:(id)sender
{
    // Make a backup file
    [[OSxCDStackManager sharedManager] backupCurrentStore];
}
- (IBAction)loadAction:(id)sender
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(jobDone) name:OSBackgroundTaskDoneNotification object:nil];

    [[OSxCDStackManager sharedManager] loadDataInBackground];
    [activityIndicator startAnimation:self];
    _taskCount++;
}
- (IBAction)deleteAction:(id)sender
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(jobDone) name:OSBackgroundTaskDoneNotification object:nil];
    [[OSxCDStackManager sharedManager] deleteDataInBackground];
    [activityIndicator startAnimation:self];
    _taskCount++;
}
- (void)jobDone {
    _taskCount--;
    
    if (_taskCount == 0) {
        [activityIndicator stopAnimation:self];
    }
}
- (BOOL)validateUserInterfaceItem:(id < NSValidatedUserInterfaceItem >)anItem {
    //FLOG(@" called for item %ld, %@", anItem.tag, anItem);
    
    // Check if the document is already in iCloud
    if ([anItem tag] == 1001) { // iCloud menu
        NSMenuItem *menuItem = (NSMenuItem *)anItem;
        if ([[OSxCDStackManager sharedManager] isCloudEnabled]) {
            [menuItem setTitle:@"Remove from iCloud"];
            return YES;
        }
        else {
            [menuItem setTitle:@"Share in iCloud"];
            return YES;
            
        }
    }
    if ([anItem tag] == 101) { // save menu
        FLOG(@" Checking if save required");
        if ([[[OSxCDStackManager sharedManager] managedObjectContext] hasChanges])
            return YES;
        else
            return NO;
    }
    if ([anItem tag] == 102) { // background load menu
        if ([[OSxCDStackManager sharedManager] managedObjectContext])
            return YES;
        else
            return NO;
    }
    if ([anItem tag] == 103) { // background delete menu
        if ([[OSxCDStackManager sharedManager] managedObjectContext])
            return YES;
        else
            return NO;
    }
    if ([anItem tag] == 104) { // backup menu
        if ([[OSxCDStackManager sharedManager] managedObjectContext])
            return YES;
        else
            return NO;
    }
    return NO;
}
- (void)outlineViewSelectionDidChange:(NSNotification *)notification {
    if ([_sidebarOutlineView selectedRow] != -1) {
        NSObject *item = [_sidebarOutlineView itemAtRow:[_sidebarOutlineView selectedRow]];
        if ([_sidebarOutlineView parentForItem:item] != nil) {
            // Only change things for non-root items (root items can be selected, but are ignored)
            [self _setContentViewToName:item];
        }
    }
}
- (IBAction)setUpgradeView:(id)sender {
    FLOG(@" called");
    if ([NSThread isMainThread]) {
        [self _setContentViewToName:@"Upgrade"];
    } else {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self _setContentViewToName:@"Upgrade"];
        });
    }
}
- (void)_setContentViewToName:(NSObject *)item  {
    if (_currentContentViewController) {
        [[_currentContentViewController view] removeFromSuperview];
    }
    _currentContentViewController = [self getContentViewFor:item]; // Retained
    NSView *view = [_currentContentViewController view];
    view.frame = _mainContentView.bounds;
    [view setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
    [_mainContentView addSubview:view];
}
- (NSViewController*)getContentViewFor:(NSObject *)item {
    //FLOG(@" called with %@", [item description]);
    
    NSString *name = [item description];
    // Set the tab view here
    if ([name isEqualToString:@"Companies"])
        return [self createCompanyView];
    
    if ([name isEqualToString:@"People"])
        return [self createPeopleView];
    
    if ([name isEqualToString:@"Upgrade"])
        return [self createUpgradeView];
    // Add observer in case the user changes some of the categories and we can update the menu
    //if ([name isEqualToString:@"Administration"]) {
    //    return [self createSetupView];
    
    
    //Oops ! should'nt get here
    FLOG(@" Oops could not find item %@", [item description]);
    return nil;
}


- (NSViewController*)createCompanyView
{
    FLOG(@" called");
    //Check if its already created (oops what if we change the category name ??) - need to retain a reference to the object id or something
    
    if (_companyViewController == nil) {
        
        _companyViewController = [[CompanyViewController alloc] initWithNibName:@"CompanyViewController" bundle:nil];
        [_companyViewController setManagedObjectContext:[[OSxCDStackManager sharedManager] managedObjectContext]];
        
    }
    return _companyViewController;
}
- (NSViewController*)createPeopleView
{
    FLOG(@" called");
    //Check if its already created (oops what if we change the category name ??) - need to retain a reference to the object id or something
    
    if (_peopleViewController == nil) {
        
        _peopleViewController = [[PeopleViewController alloc] initWithNibName:@"PeopleViewController" bundle:nil];
        [_peopleViewController setManagedObjectContext:[[OSxCDStackManager sharedManager] managedObjectContext]];
        
    }
    return _peopleViewController;
}
- (NSViewController*)createUpgradeView
{
    FLOG(@" called");
    //Check if its already created (oops what if we change the category name ??) - need to retain a reference to the object id or something
    
    if (_upgradeViewController == nil) {
        
        _upgradeViewController = [[UpgradeViewController alloc] initWithNibName:@"UpgradeViewController" bundle:nil];
        [_upgradeViewController setManagedObjectContext:[[OSxCDStackManager sharedManager] managedObjectContext]];
        
    }
    return _upgradeViewController;
}

- (void)saveWindowPosition {
    //LOG(@"saveWindowPosition called");
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSString *autosaveNameKey = [NSString stringWithFormat:@"Main Frame"];
    [defaults setObject:[self stringFromFrame] forKey:autosaveNameKey];
    //[self printRect:[self rectFromString:[self stringFromFrame]]];
}
- (void)restoreSavedWindowPosition {
    //LOG(@"restoreSavedWindowPosition called");
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSString *autosaveNameKey = [NSString stringWithFormat:@"Main Frame"];
    NSString *frameString = [defaults objectForKey:autosaveNameKey];
    
    // Do nothing if it has not been set
    if (frameString) {
        //[self printRect:[self rectFromString:frameString]];
        [self.window setFrame:[self rectFromString:frameString] display:YES animate:YES];
    }
    else {
        // Set the default for new docs
        NSRect rect = CGRectMake(70, 350, 1000, 760);
        //[self printRect:rect];
        [self.window setFrame:rect display:YES animate:YES];
    }
    
    return;
}
- (NSString *)stringFromFrame {
    NSRect rect = self.window.frame;
    rect.origin.y = rect.origin.y + 26;
    rect.size.height = rect.size.height - 26;
    
    return [self stringFromRect:rect];
}
- (NSString *)stringFromRect:(NSRect)rect {
    return [NSString stringWithFormat:@"%f %f %f %f", rect.origin.x, rect.origin.y, rect.size.width, rect.size.height];
}
- (NSRect)rectFromString:(NSString*)string {
    NSRect rect;
    NSArray *array = [string componentsSeparatedByString:@" "];
    rect.origin.x = [[array objectAtIndex:0] floatValue];
    rect.origin.y = [[array objectAtIndex:1] floatValue];
    rect.size.width = [[array objectAtIndex:2] floatValue];
    rect.size.height = [[array objectAtIndex:3] floatValue];
    return rect;
}
@end

@implementation MainWindowController (NSOutlineViewDataSource)
- (NSArray *)_childrenForItem:(id)item {
    //FLOG(@" called");
    NSArray *children;
    if (item == nil) {
        children = _topLevelItems;
    } else {
        children = [_childrenDictionary objectForKey:item];
    }
    return children;
}

- (id)outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item {
    //FLOG(@" called");
    return [[self _childrenForItem:item] objectAtIndex:index];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item {
    //FLOG(@" called");
    if ([outlineView parentForItem:item] == nil) {
        return YES;
    } else {
        return NO;
    }
}

- (NSInteger) outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item {
    //FLOG(@" called");
    return [[self _childrenForItem:item] count];
}
@end

@implementation MainWindowController (NSOutlineViewDelegate)

- (BOOL)outlineView:(NSOutlineView *)outlineView isGroupItem:(id)item {
    return [_topLevelItems containsObject:item];
}
- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectItem:(id)item
{
    // Don't allow menu selection until store has been initialised
    if (!_storeIsReady)
        return NO;
    
    if ([_topLevelItems containsObject:item])
        return NO;
    else
        return YES;
}
- (BOOL)outlineView:(NSOutlineView *)outlineView shouldShowOutlineCellForItem:(id)item {
    // As an example, hide the "outline disclosure button" for FAVORITES. This hides the "Show/Hide" button and disables the tracking area for that row.
    if ([item isEqualToString:@"Favorites"]) {
        return NO;
    } else {
        return YES;
    }
}


- (NSView *)outlineView:(NSOutlineView *)outlineView viewForTableColumn:(NSTableColumn *)tableColumn item:(id)item {
    // For the groups, we just return a regular text view.
    if ([_topLevelItems containsObject:item]) {
        NSTextField *result = [outlineView makeViewWithIdentifier:@"HeaderTextField" owner:self];
        // Uppercase the string value, but don't set anything else. NSOutlineView automatically applies attributes as necessary
        NSString *value = [item uppercaseString];
        [result setStringValue:value];
        return result;
    } else  {
        // The cell is setup in IB. The textField and imageView outlets are properly setup.
        // Special attributes are automatically applied by NSTableView/NSOutlineView for the source list
        OSSidebarTableCellView *result = [outlineView makeViewWithIdentifier:@"MainCell" owner:self];
        result.textField.stringValue = [item description];
        // Setup the icon based on our section
        //id parent = [outlineView parentForItem:item];
        //NSInteger index = [_topLevelItems indexOfObject:parent];
        //NSInteger iconOffset = index % 4;
        
        
        result.imageView.image = [self itemImage:item];
        
        /*
         switch (iconOffset) {
         case 0: {
         result.imageView.image = [NSImage imageNamed:NSImageNameIconViewTemplate];
         break;
         }
         case 1: {
         result.imageView.image = [NSImage imageNamed:NSImageNameHomeTemplate];
         break;
         }
         case 2: {
         result.imageView.image = [NSImage imageNamed:NSImageNameQuickLookTemplate];
         break;
         }
         case 3: {
         result.imageView.image = [NSImage imageNamed:NSImageNameSlideshowTemplate];
         break;
         }
         }
         */
        BOOL hideUnreadIndicator = YES;
        /*
         // Setup the unread indicator to show in some cases. Layout is done in SidebarTableCellView's viewWillDraw
         if (index == 0) {
         // First row in the index
         hideUnreadIndicator = YES;
         [result.button setTitle:@"42"];
         [result.button sizeToFit];
         // Make it appear as a normal label and not a button
         [[result.button cell] setHighlightsBy:0];
         } else if (index == 2) {
         // Example for a button
         hideUnreadIndicator = NO;
         result.button.target = self;
         result.button.action = @selector(buttonClicked:);
         [result.button setImage:[NSImage imageNamed:NSImageNameAddTemplate]];
         // Make it appear as a button
         [[result.button cell] setHighlightsBy:NSPushInCellMask|NSChangeBackgroundCellMask];
         }
         */
        [result.button setHidden:hideUnreadIndicator];
        return result;
    }
}

@end

@implementation MainWindowController (NSSplitViewDelegate)
- (BOOL)splitView:(NSSplitView *)splitView canCollapseSubview:(NSView *)subview {
    return NO;
}

- (CGFloat)splitView:(NSSplitView *)splitView constrainMinCoordinate:(CGFloat)proposedMinimumPosition ofSubviewAt:(NSInteger)dividerIndex {
    if (proposedMinimumPosition < 75) {
        proposedMinimumPosition = 75;
    }
    return proposedMinimumPosition;
}
- (BOOL)splitView:(NSSplitView *)splitView shouldAdjustSizeOfSubview:(NSView *)subview {
    if (subview == leftview) return NO;
    else return YES;
}
@end