//
//  OSxCDStackManager.h
//  Common
//
//  Created by Duncan Groenewald on 15/07/13.
//  Copyright (c) 2013 Duncan Groenewald. All rights reserved.
//

#define FLOG(format, ...)               NSLog(@"%@.%@ %@", [self class], NSStringFromSelector(_cmd), [NSString stringWithFormat:format, ##__VA_ARGS__])

#define STORE_KEEP    0
#define STORE_MOVE    1
#define STORE_DELETE  2

@interface FileRepresentation : NSObject

@property (nonatomic, readonly) NSString* fileName;
@property (nonatomic, readonly) NSURL* url;
@property (nonatomic, retain) NSURL* previewURL;
@property (nonatomic, readonly) NSString* fileDate;
@property (nonatomic, readonly) NSNumber* percentDownloaded;
@property (retain) NSNumber* isDownloaded;
@property  bool ready;

- (id)initWithFileName:(NSString*)fileName url:(NSURL*)url;
- (id)initWithFileName:(NSString*)fileName url:(NSURL*)url date:(NSString*)fileDate;
- (id)initWithFileName:(NSString*)fileName url:(NSURL*)url percentDownloaded:(NSNumber*)percent;
- (NSString*)modifiedDate;

@end

// CloudManager performs file management asynchronously so we need callbacks to let the requester know when we are done
@protocol OSxCDStackManagerProtocol <NSObject>
@required
-(void)fileHasBeenOpened:(NSURL*)file;
-(void)fileHasBeenCreated:(NSURL*)file;
-(void)fileHasBeenDeleted:(NSURL*)file;
-(void)fileHasBeenClosed:(NSURL*)file;
@end

@interface OSxCDStackManager : NSObject {
    
    BOOL _useICloudStorage;

    NSArray * _localURLs;
    NSArray * _iCloudURLs;
    NSArray* _localDocuments;

    BOOL _isBusy;
    bool _isOpening;
    BOOL _isMigratingToICloud;
    BOOL _isMigratingToLocal;
    NSURL *_migratingFileURL;
    BOOL _import_or_save;
    BOOL _load_seed_data;
    BOOL _use_old_model;
    BOOL _isUpgrading;
    
    bool _deletingDocument;
    bool _creatingDocument;
    bool _creatingNewFile;

    int _migrationCounter;
    
    bool _clearFiles;  // set this to delete all files on the device on startup
    
    bool _storesChanging;
    
    bool _startUpgrade;
    
    //UIAlertView* _storesUpdatingAlert;
    //UIAlertView* _cloudChoiceAlert;
    //UIAlertView* _cloudChangedAlert;
    //UIAlertView* _migratingAlert;
    //UIAlertView* _cloudMergeChoiceAlert;

    NSManagedObjectModel *_managedObjectModel;
    NSManagedObjectContext *_managedObjectContext;
    NSPersistentStoreCoordinator *_persistentStoreCoordinator;
    NSURL *_storeURL;

    NSMetadataQuery* _query;
    NSManagedObjectModel *_sourceModel;  // Used if an Cloud upgrade is required

    long _companyCount;
    long _personCount;
}

@property (readonly, strong, nonatomic) NSManagedObjectContext *managedObjectContext;
@property (readonly, strong, nonatomic) NSManagedObjectModel *managedObjectModel;
@property (readonly, strong, nonatomic) NSPersistentStoreCoordinator *persistentStoreCoordinator;
@property (readonly, strong, nonatomic) NSURL *persistentStoreName;
@property (strong, nonatomic) NSURL *storeURL;

- (void)saveContext;
- (NSURL *)applicationFilesDirectory;
- (NSArray *)getData:(NSString*)entityName sortField:(NSString*)sortKey predicate:(NSPredicate*)predicate;

@property (readwrite, retain)   NSTimer *iCloudUpdateTimer;

@property (nonatomic)           BOOL isCloudEnabled;
@property (nonatomic)           BOOL deleteICloudFiles;
@property (nonatomic, readonly) NSURL* dataDirectoryURL;
@property (nonatomic, readonly) NSURL* documentsDirectoryURL;
@property (nonatomic)           NSString* ubiquityID;
@property (nonatomic, strong)   NSString *cloudPreferenceKey;
@property (nonatomic, strong)   NSString *cloudPreferenceSet;
@property (nonatomic, strong)   NSString *makeBackupPreferenceKey;
@property (nonatomic, strong)   NSString *loadInBackgroundPreferenceKey;
@property (nonatomic, strong)   NSString *ubiquityContainerKey;
@property (nonatomic, strong)   NSString *ubiquityIDToken;
@property (nonatomic)           bool startUpgrade;


+ (OSxCDStackManager*)sharedManager;
- (void)setUbiquityID:(NSString*)ubiquityID;
- (void)checkUserICloudPreferenceAndSetupIfNecessary;
- (void)performApplicationWillEnterForegroundCheck;
- (void)loadDataInBackground;
- (void)deleteDataInBackground;
- (void)setICloudPreference:(bool)pref;
- (bool)backupCurrentStore;

- (NSURL*)iCloudCoreDataURL;
- (bool)canUseICloud;
- (bool)isICloudAvailable;
- (bool)isBusy;

- (void)saveDocument;
- (void)closeDocument;
- (void)postUIUpdateNotification;

- (NSDictionary*)icloudStoreOptions;
- (NSDictionary*)localStoreOptions;
- (bool)moveStoreToLocal;
- (bool)moveStoreToICloud;

- (void)saveFile:(NSURL *)fileURL;


@end

NSString* const ICloudStateUpdatedNotification;

NSString* const OSFileDeletedNotification;
NSString* const OSFileCreatedNotification;
NSString* const OSFileClosedNotification;
NSString* const OSFilesUpdatedNotification;
NSString* const OSDataUpdatedNotification;
NSString* const OSStoreChangeNotification;
NSString* const OSStoreReadyNotification;
NSString* const OSStoreUpgradingNotification;
NSString* const OSStoreUpgradeStatusNotification;
NSString* const OSBackgroundTaskDoneNotification;
